;; See the file LICENCE for licence information.
(in-package :ucw)

(eval-always
  (shadowing-import '(cl-l10n:locale cl-l10n:locale-name) :ucw)
  (use-package :cl-l10n :ucw)
  (use-package :cl-l10n :ucw-user))

(defun reload-ucw-resources ()
  "This method reloads the UCW resources needed by some UCW services. Invoke
   after your :lang package has been set up."
  (flet ((system-relative-pathname (system path)
           (merge-pathnames path (asdf:component-pathname
                                  (asdf:find-system system)))))
    (let ((resource-dir (system-relative-pathname
                         :ucw (make-pathname :directory (list :relative "src" "l10n" "resources")
                                             :name :wild
                                             :type "lisp"))))
      (ucw.l10n.debug "Looking for UCW resources in dir ~S" resource-dir)
      (dolist (resource-file (directory resource-dir))
        (ucw.l10n.info "Loading UCW resource file ~S" resource-file)
        (load resource-file)))))

(defmacro define-js-resources (locale &body resources)
  "Wraps defresources and registers the resources being defined
as js resources, so that they are sent down to the client side."
  `(progn
    (register-js-resources
     ,@(iter (for (name . body) in resources)
             (when (> (length body) 1)
               (error "How should we send functional resources to js? ~S has arguments..." name))
             (collect `',name)))
    (defresources ,locale ,@resources)))

(defclass l10n-tal-generator (file-system-generator)
  ()
  (:documentation "TAL File system generator which maps template
  names to files while considering the locale of the request
  context."))

(defmethod yaclml:template-truename ((generator l10n-tal-generator) (name pathname))
  ;; first see if there's a locale specific version
  (when-bind locales (context.locale *context*)
    (ucw.l10n.debug "Looking for tal directory with locales ~A" locales)
    (iter (for locale in locales)
          (for file = (make-pathname :defaults name
                                     :directory (list :relative (locale-name locale))))
          (ucw.l10n.debug "Trying tal file ~A" file)
          (awhen (call-next-method generator file)
            (ucw.l10n.debug "Found locale specific tal directory ~A" it)
            (return-from yaclml:template-truename it))))
  ;; no locale for this context, use the default
  (ucw.l10n.debug "No locale specific tal directory found for locale file ~A" name)
  (call-next-method))

;;;
;;; Some reader syntax to help l10n
;;;
(deftag-macro <ucw:localized (&attribute (warn-if-missing t) args &body forms)
  "Just like <:as-html but calls localize on the elements before rendering and
   if a resource cannot be found then wraps it in a <:span with a css class."
  `(progn
    ,@(mapcar (lambda (form)
                `(multiple-value-bind (str foundp) (lookup-resource ,form ,args
                                                    :warn-if-missing ,warn-if-missing
                                                    :fallback-to-name t)
                  (if foundp
                      (<:as-html str)
                      (<:span :class #.+missing-resource-css-class+
                              (<:as-html str)))))
              forms)))

(defun dojo-locale-name-for (locale)
  (substitute #\- #\_
              (string-downcase
               (locale-name locale))))

;;;; * i18n-parenscript-dispatcher

(defclass i18n-parenscript-dispatcher (dispatcher url-matcher parenscript-handler)
  ((locale-to-js-map :initform (make-hash-table :test 'equal) :accessor locale-to-js-map-of)
   (last-purged-timestamp :initform (get-universal-time) :accessor last-purged-timestamp-of)
   (purge-interval :initform #.(* 60 60 24) :initarg :purge-interval :accessor purge-interval-of))
  (:default-initargs
     :priority +parenscript-dispatcher-default-priority+
     :url-string +i18n-parenscript-dispatcher-url+)
  (:documentation "This handler is used to serve a
compiled javascript string from *js-resource-registry*."))

(defmethod parenscript-handler-timestamp ((self i18n-parenscript-dispatcher))
  *js-resource-registry-last-modified*)

(defmethod parenscript-handler-compile ((self i18n-parenscript-dispatcher))
  (let* ((forms `(ucw.i18n.define
                  ,@(iter (for (name) in-hashtable *js-resource-registry*)
                          (collect (string-downcase (string name)))
                          (collect (localize name)))))
         (result (js:js* forms)))
    (ucw.rerl.dispatcher.info "~S is compiling for key ~S" self (i18n-parenscript-dispatcher-cache-key))
    (ucw.rerl.dispatcher.debug "~S at key ~S compiled ~S" self (i18n-parenscript-dispatcher-cache-key) result)
    result))

(defun i18n-parenscript-dispatcher-cache-key ()
  (mapcar #'locale-name (ensure-list *locale*)))

(defmethod cached-javascript ((self i18n-parenscript-dispatcher))
  (let* ((key (i18n-parenscript-dispatcher-cache-key))
         (result (gethash key (locale-to-js-map-of self))))
    (ucw.rerl.dispatcher.debug "~S ~A cached js with key ~S"
                               self (if result "found" "missed") key)
    result))

(defmethod (setf cached-javascript) (value (self i18n-parenscript-dispatcher))
  (when (> (get-universal-time)
           (+ (last-purged-timestamp-of self)
              (purge-interval-of self)))
    (ucw.rerl.dispatcher.info "~S is purging its cache" self))
  (let ((key (i18n-parenscript-dispatcher-cache-key)))
    (ucw.rerl.dispatcher.debug "~S is storing cached js with key ~S" self key)
    (setf (gethash key (locale-to-js-map-of self)) value)))


;; let's redefine the old ucw stubs, so that already expanded #""'s will work, too
(defun ucw-lookup-resource (name args)
  (lookup-resource name args))
(defun ucw-capitalize-first-letter (str)
  (capitalize-first-letter str))

;; and change the functions to be the cl-l10n versions for later expansions of the reader macro
(setf *sharpquote-resource-lookup-function* 'lookup-resource)
(setf *sharpquote-capitalize-first-letter-function* 'capitalize-first-letter)
